#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ordine.h"
#include "readField.h"

Ordine* leggiOrdini(char fileName[], int *dim)
{
	FILE *f;
	Boolean ok;
	Ordine *ordini;

	f = fopen("ordini.bin", "rb");
	if (f == NULL)
	{
		return NULL;
	}

	ok = fread(dim, sizeof(int), 1, f) == 1;

	if (ok)
	{
		ordini = (Ordine*)
			malloc(sizeof(Ordine) * (*dim));

		ok = fread(ordini, sizeof(Ordine), *dim, f) == *dim;		
	}
	fclose(f);
	if (ok)
	{
		return ordini;
	}
	else
	{
		return NULL;
	}
}

void printOrdini(Ordine ordini[], int dim)
{
	int i;
	for (i = 0; i < dim; i++)
	{
		printf("%s -- %d\n", ordini[i].nomeCliente, ordini[i].numIngredienti);
	}
}


float calcolaPrezzo(Ordine o, IngredienteList list)
{
	float prezzo = 0;
	int i;
	for (i = 0; i < o.numIngredienti; i++)
	{
		prezzo += trovaPrezzo(list, o.ings[i].nome);
	}
	return prezzo;
}

void swap(Ordine *o1, Ordine *o2)
{
	Ordine tmp;
	tmp = *o1;
	*o1 = *o2;
	*o2 = tmp;
}

void sortOrdini(Ordine ordini[], int dim)
{	
    int  j, i, posmin;
    Ordine min;

    for  (j = 0; j < dim; j++)
    {
        posmin = j;
        for (min = ordini[j], i = j + 1; i < dim; i++)
			if (strcmp(ordini[i].nomeCliente,  min.nomeCliente) < 0)
            {
                posmin = i;
                min = ordini[i];
            }

        if (posmin != j)
            swap(&ordini[j], &ordini[posmin]);
    }
}

void printPrezzoPerCliente(Ordine ordini[], int dim, 
						   IngredienteList list)
{
	char corrente[MAX_NOME_CLIENTE] = "";
	float totaleCorrente;
	int i;

	for (i = 0; i < dim; i++)
	{
		if (strcmp(ordini[i].nomeCliente, corrente) == 0)
		{
			totaleCorrente += calcolaPrezzo(ordini[i], list);
		}
		else
		{
			if (i != 0)
			{
				printf("%s -- %.2fSoldi\n", corrente, totaleCorrente);
			}
			totaleCorrente = calcolaPrezzo(ordini[i], list);
			strcpy(corrente, ordini[i].nomeCliente);
		}
	}
	printf("%s -- %.2fSoldi\n", corrente, totaleCorrente);
}

Boolean readOrdineFromTxt(FILE *f, Ordine *o)
{
	Boolean ok;
	Boolean ingredienteOk;
	int i;

	ok = readField(f, o->nomeCliente, ';') == ';';

	if (ok)
	{
		Boolean continuaLettura = true;
		ingredienteOk = true;
		i = 0;	
		while(ingredienteOk && continuaLettura)
		{
			ingredienteOk = readField(f, o->ings[i].nome, ',') == ',';
			ingredienteOk = ingredienteOk && 
				fscanf(f, "%f", &o->ings[i].prezzo) == 1;
			
			if (ingredienteOk)
			{
				continuaLettura = fgetc(f) == ';';
				i++;
			}			
		}
		ok = ingredienteOk;
		if (ok)
		{
			o->numIngredienti = i;
		}
	}

	return ok;
}

Ordine* leggiOrdiniFromTxt(char fileName[], int *dim)
{
	int count;
	int i;
	FILE *f;
	Ordine o;
	Ordine *ordini = NULL;

	f = fopen(fileName, "r");
	if (f == NULL)
	{
		return ordini;
	}

	count = 0;
	while (readOrdineFromTxt(f, &o))
	{
		count++;
	}

	*dim = count;
	ordini = (Ordine*)malloc(sizeof(Ordine) * count);
	rewind(f);

	for (i = 0; i < count; i++)
	{
		readOrdineFromTxt(f, ordini + i);
	}

	return ordini;
}